@@ -22,9 +22,17 @@
  *
  */
  
-#include <stdlib.h>
-#include <string.h>
+#include <stdlib.h> /* malloc */
+#include <string.h> /* strlen,  strdup */
+#include <unistd.h> /* lseek, close */
 #include <locale.h>
+#include <errno.h>
+
+#include <fcntl.h>
+#include <sys/mman.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <pwd.h>
 
 #include <gtk/gtk.h>
 
@@ -32,6 +40,10 @@
 #include "parse.h"
 
 #include "ctkdisplayconfig-utils.h"
+#include "ctkutils.h"
+
+
+static void xconfig_update_buffer(GtkWidget *widget, gpointer user_data);
 
 
 
@@ -2548,3 +2560,742 @@ nvScreenPtr layout_get_a_screen(nvLayoutPtr layout, nvGpuPtr preferred_gpu)
     return screen;
 
 } /* layout_get_a_screen() */
+
+
+
+
+/*****************************************************************************/
+/** XCONFIG FUNCTIONS ********************************************************/
+/*****************************************************************************/
+
+
+/*
+ * tilde_expansion() - do tilde expansion on the given path name;
+ * based loosely on code snippets found in the comp.unix.programmer
+ * FAQ.  The tilde expansion rule is: if a tilde ('~') is alone or
+ * followed by a '/', then substitute the current user's home
+ * directory; if followed by the name of a user, then substitute that
+ * user's home directory.
+ *
+ * Code adapted from nvidia-xconfig
+ */
+
+char *tilde_expansion(char *str)
+{
+    char *prefix = NULL;
+    char *replace, *user, *ret;
+    struct passwd *pw;
+    int len;
+
+    if ((!str) || (str[0] != '~')) return str;
+    
+    if ((str[1] == '/') || (str[1] == '\0')) {
+
+        /* expand to the current user's home directory */
+
+        prefix = getenv("HOME");
+        if (!prefix) {
+            
+            /* $HOME isn't set; get the home directory from /etc/passwd */
+            
+            pw = getpwuid(getuid());
+            if (pw) prefix = pw->pw_dir;
+        }
+        
+        replace = str + 1;
+        
+    } else {
+    
+        /* expand to the specified user's home directory */
+
+        replace = strchr(str, '/');
+        if (!replace) replace = str + strlen(str);
+
+        len = replace - str;
+        user = malloc(len + 1);
+        strncpy(user, str+1, len-1);
+        user[len] = '\0';
+        pw = getpwnam(user);
+        if (pw) prefix = pw->pw_dir;
+        free (user);
+    }
+
+    if (!prefix) return str;
+    
+    ret = malloc(strlen(prefix) + strlen(replace) + 1);
+    strcpy(ret, prefix);
+    strcat(ret, replace);
+    
+    return ret;
+
+} /* tilde_expansion() */
+
+
+
+/*
+ * update_banner() - add our banner at the top of the config, but
+ * first we need to remove any lines that already include our prefix
+ * (because presumably they are a banner from an earlier run of
+ * nvidia-settings)
+ *
+ * Code adapted from nvidia-xconfig
+ */
+
+extern const char *pNV_ID;
+
+static void update_banner(XConfigPtr config)
+{
+    static const char *banner =
+        "X configuration file generated by nvidia-settings\n";
+    static const char *prefix =
+        "# nvidia-settings: ";
+
+    char *s = config->comment;
+    char *line, *eol, *tmp;
+    
+    /* remove all lines that begin with the prefix */
+    
+    while (s && (line = strstr(s, prefix))) {
+        
+        eol = strchr(line, '\n'); /* find the end of the line */
+        
+        if (eol) {
+            eol++;
+            if (*eol == '\0') eol = NULL;
+        }
+        
+        if (line == s) { /* the line with the prefix is at the start */
+            if (eol) {   /* there is more after the prefix line */
+                tmp = g_strdup(eol);
+                g_free(s);
+                s = tmp;
+            } else {     /* the prefix line is the only line */
+                g_free(s);
+                s = NULL;
+            }
+        } else {         /* prefix line is in the middle or end */
+            *line = '\0';
+            tmp = g_strconcat(s, eol, NULL);
+            g_free(s);
+            s = tmp;
+        }
+    }
+    
+    /* add our prefix lines at the start of the comment */
+    config->comment = g_strconcat(prefix, banner,
+                                  "# ", pNV_ID, "\n",
+                                  (s ? s : ""),
+                                  NULL);
+    if (s) g_free(s);
+    
+} /* update_banner() */
+
+
+
+/** save_xconfig_file() **********************************************
+ *
+ * Saves the X config file text from buf into a file called
+ * filename.  If filename already exists, a backup file named
+ * 'filename.backup' is created.
+ *
+ **/
+
+static int save_xconfig_file(SaveXConfDlg *dlg,
+                             gchar *filename, char *buf, mode_t mode)
+{
+    gchar *backup_filename = NULL;
+    FILE *fp = NULL;
+    size_t size;
+    gchar *err_msg = NULL;
+
+    int ret = 0;
+
+
+    if (!buf || !filename) goto done;
+
+    size = strlen(buf) ;
+
+    /* Backup any existing file */
+    if ((access(filename, F_OK) == 0)) {
+
+        backup_filename = g_strdup_printf("%s.backup", filename);
+        nv_info_msg("", "X configuration file '%s' already exists, "
+                    "backing up file as '%s'", filename,
+                    backup_filename);
+        
+        /* Delete any existing backup file */
+        if (access(backup_filename, F_OK) == 0) {
+
+            if (unlink(backup_filename) != 0) {
+                err_msg =
+                    g_strdup_printf("Unable to remove old X config backup "
+                                    "file '%s'.",
+                                    backup_filename);
+                goto done;
+            }
+        }
+
+        /* Make the current x config file the backup */
+        if (rename(filename, backup_filename)) {
+                err_msg =
+                    g_strdup_printf("Unable to create new X config backup "
+                                    "file '%s'.",
+                                    backup_filename);
+            goto done;            
+        }
+    }
+
+    /* Write out the X config file */
+    fp = fopen(filename, "w");
+    if (!fp) {
+        err_msg =
+            g_strdup_printf("Unable to open X config file '%s' for writing.",
+                            filename);
+        goto done;
+    }
+    fprintf(fp, "%s", buf);
+
+    ret = 1;
+    
+ done:
+    /* Display any errors that might have occured */
+    if (err_msg) {
+        ctk_display_error_msg(ctk_get_parent_window(GTK_WIDGET(dlg->parent)),
+                              err_msg);
+        g_free(err_msg);
+    }
+
+    if (fp) fclose(fp);
+    g_free(backup_filename);
+    return ret;
+    
+} /* save_xconfig_file() */
+
+
+
+/**  update_xconfig_save_buffer() ************************************
+ *
+ * Updates the "preview" buffer to hold the right contents based on
+ * how the user wants the X config file to be generated (and what is
+ * possible.)
+ *
+ * Also updates the state of the "Merge" checkbox in the case where
+ * the named file can/cannot be parsed as a valid X config file.
+ *
+ */
+static void update_xconfig_save_buffer(SaveXConfDlg *dlg)
+{
+    gchar *filename;
+
+    XConfigPtr xconfCur = NULL;
+    XConfigPtr xconfGen = NULL;
+    XConfigError xconfErr;
+
+    char *tmp_filename;
+    int tmp_fd;
+    struct stat st;
+    void *buf;
+    GtkTextIter buf_start, buf_end;
+
+    gboolean merge;
+    gboolean merged;
+    gboolean merge_error;
+
+    gchar *err_msg = NULL;
+
+
+
+    /* Get how the user wants to generate the X config file */
+    merge = gtk_toggle_button_get_active
+        (GTK_TOGGLE_BUTTON(dlg->btn_xconfig_merge));
+
+    filename = (gchar *)gtk_entry_get_text(GTK_ENTRY(dlg->txt_xconfig_file));
+
+    /* Find out if the file is mergable */
+    if (filename && (filename[0] != '\0')) {
+        /* Must be able to open the file */
+        tmp_filename = (char *)xconfigOpenConfigFile(filename, NULL);
+        if (!tmp_filename || strcmp(tmp_filename, filename)) {
+            err_msg = g_strdup_printf("Failed to open existing X config "
+                                      "file '%s'!",
+                                      filename);
+            ctk_display_warning_msg
+                (ctk_get_parent_window(GTK_WIDGET(dlg->parent)), err_msg);
+
+            xconfigCloseConfigFile();
+
+        } else {
+            GenerateOptions gop;
+
+            /* Must be able to parse the file as an X config file */
+            xconfErr = xconfigReadConfigFile(&xconfCur);
+            xconfigCloseConfigFile();
+            if ((xconfErr != XCONFIG_RETURN_SUCCESS) || !xconfCur) {
+                /* If we failed to parse the config file, we should not
+                 * allow a merge.
+                 */
+                err_msg = g_strdup_printf("Failed to parse existing X "
+                                          "config file '%s'!",
+                                          filename);
+                ctk_display_warning_msg
+                    (ctk_get_parent_window(GTK_WIDGET(dlg->parent)), err_msg);
+
+                xconfCur = NULL;
+            }
+
+            /* Sanitize the X config file */
+            xconfigGenerateLoadDefaultOptions(&gop);
+            xconfigGetXServerInUse(&gop);
+
+            if (!xconfigSanitizeConfig(xconfCur, NULL, &gop)) {
+                err_msg = g_strdup_printf("Failed to sanitize existing X "
+                                          "config file '%s'!",
+                                          filename);
+                ctk_display_warning_msg
+                    (ctk_get_parent_window(GTK_WIDGET(dlg->parent)), err_msg);
+
+                xconfigFreeConfig(&xconfCur);
+                xconfCur = NULL;
+            }
+        }
+
+        /* If we're not actualy doing a merge, close the file */
+        if (!merge && xconfCur) {
+            xconfigFreeConfig(&xconfCur);
+        }
+    }
+
+    merge_error = merge && !xconfCur;
+    merge = (merge && !merge_error);
+
+
+    /* If we have to merge but we can't, report the problem. */
+    if (merge_error && !dlg->merge_toggleable) {
+        // XXX Instead of bailing here, we may want to just disable
+        //     the "Save" button until the user specifies a file
+        //     that we can parse.
+        goto fail;
+    }
+
+    /* Generate the X config file */
+    xconfGen = dlg->xconf_gen_func(xconfCur, merge, &merged,
+                                   dlg->callback_data);
+    if (!xconfGen) {
+        err_msg = g_strdup_printf("Failed to generate X config file!");
+        goto fail;
+    }
+
+    merge_error = merge && !merged;
+
+    /* Update merge status */
+    g_signal_handlers_block_by_func
+        (G_OBJECT(dlg->btn_xconfig_merge),
+         G_CALLBACK(xconfig_update_buffer), (gpointer) dlg);
+    
+    gtk_toggle_button_set_active
+        (GTK_TOGGLE_BUTTON(dlg->btn_xconfig_merge), merged);
+    
+    g_signal_handlers_unblock_by_func
+        (G_OBJECT(dlg->btn_xconfig_merge),
+         G_CALLBACK(xconfig_update_buffer), (gpointer) dlg);
+    
+    gtk_widget_set_sensitive(dlg->btn_xconfig_merge,
+                             (dlg->merge_toggleable && !merge_error) ?
+                             TRUE : FALSE);
+    
+
+    /* We're done with the user's X config, so do some cleanup,
+     * but make sure to handle the case where the generation
+     * function modifies the user's X config structure instead
+     * of generating a new one.
+     */
+    if (xconfGen == xconfCur) {
+        xconfCur = NULL;
+    } else {
+        xconfigFreeConfig(&xconfCur);
+    }
+
+
+    /* Update the X config banner */
+    update_banner(xconfGen);
+
+
+    /* Setup the X config file preview buffer by writing to a temp file */
+    tmp_filename = g_strdup_printf("/tmp/.xconfig.tmp.XXXXXX");
+    tmp_fd = mkstemp(tmp_filename);
+    if (!tmp_fd) {
+        err_msg = g_strdup_printf("Failed to create temp X config file '%s' "
+                                  "for display.",
+                                  tmp_filename);
+        g_free(tmp_filename);
+        goto fail;
+    }
+    xconfigWriteConfigFile(tmp_filename, xconfGen);
+    xconfigFreeConfig(&xconfGen);
+
+    lseek(tmp_fd, 0, SEEK_SET);
+    fstat(tmp_fd, &st);
+    buf = mmap(0, st.st_size, PROT_READ, MAP_PRIVATE, tmp_fd, 0);
+
+    /* Clear the GTK buffer */
+    gtk_text_buffer_get_bounds
+        (GTK_TEXT_BUFFER(dlg->buf_xconfig_save), &buf_start,
+         &buf_end);
+    gtk_text_buffer_delete
+        (GTK_TEXT_BUFFER(dlg->buf_xconfig_save), &buf_start,
+         &buf_end);
+
+    /* Set the new GTK buffer contents */
+    gtk_text_buffer_set_text(GTK_TEXT_BUFFER(dlg->buf_xconfig_save),
+                             buf, st.st_size);
+    munmap(buf, st.st_size);
+    close(tmp_fd);
+    remove(tmp_filename);
+    g_free(tmp_filename);
+
+    return;
+
+
+ fail:
+    /* Clear the GTK buffer */
+    gtk_text_buffer_get_bounds
+        (GTK_TEXT_BUFFER(dlg->buf_xconfig_save), &buf_start,
+         &buf_end);
+    gtk_text_buffer_delete
+        (GTK_TEXT_BUFFER(dlg->buf_xconfig_save), &buf_start,
+         &buf_end);
+
+    if (err_msg) {
+        ctk_display_warning_msg(ctk_get_parent_window(GTK_WIDGET(dlg->parent)),
+                                err_msg);
+        g_free(err_msg);
+    }
+
+    if (xconfGen) {
+        xconfigFreeConfig(&xconfGen);
+    }
+
+    if (xconfCur) {
+        xconfigFreeConfig(&xconfCur);
+    }
+
+    return;
+
+} /* update_xconfig_save_buffer() */
+
+
+
+/** xconfig_preview_clicked() ****************************************
+ *
+ * Called when the user clicks on the "Preview" button of the
+ * X config save dialog.
+ *
+ **/
+
+static void xconfig_preview_clicked(GtkWidget *widget, gpointer user_data)
+{
+    SaveXConfDlg *dlg = (SaveXConfDlg *)user_data;
+    gboolean show = !GTK_WIDGET_VISIBLE(dlg->box_xconfig_save);
+
+    if (show) {
+        gtk_widget_show_all(dlg->box_xconfig_save);
+        gtk_window_set_resizable(GTK_WINDOW(dlg->dlg_xconfig_save),
+                                 TRUE);
+        gtk_widget_set_size_request(dlg->txt_xconfig_save, 450, 350);
+        gtk_button_set_label(GTK_BUTTON(dlg->btn_xconfig_preview),
+                             "Hide Preview...");
+    } else {
+        gtk_widget_hide(dlg->box_xconfig_save);
+        gtk_window_set_resizable(GTK_WINDOW(dlg->dlg_xconfig_save), FALSE);
+        gtk_button_set_label(GTK_BUTTON(dlg->btn_xconfig_preview),
+                             "Show Preview...");
+    }
+
+} /* xconfig_preview_clicked() */
+
+
+
+/** xconfig_update_buffer() ******************************************
+ *
+ * Called when the user selects a new X config filename.
+ *
+ **/
+
+static void xconfig_update_buffer(GtkWidget *widget, gpointer user_data)
+{
+    SaveXConfDlg *dlg = (SaveXConfDlg *)user_data;
+
+    update_xconfig_save_buffer(dlg);
+
+} /* xconfig_update_buffer() */
+
+
+
+/** xconfig_file_clicked() *******************************************
+ *
+ * Called when the user clicks on the "Browse..." button of the
+ * X config save dialog.
+ *
+ **/
+
+static void xconfig_file_clicked(GtkWidget *widget, gpointer user_data)
+{
+    SaveXConfDlg *dlg = (SaveXConfDlg *)user_data;
+    const gchar *filename =
+        gtk_entry_get_text(GTK_ENTRY(dlg->txt_xconfig_file));
+    gint result;
+
+
+    /* Ask user for a filename */
+    gtk_window_set_transient_for
+        (GTK_WINDOW(dlg->dlg_xconfig_file),
+         GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(dlg->parent))));
+
+    gtk_file_selection_set_filename
+        (GTK_FILE_SELECTION(dlg->dlg_xconfig_file), filename);
+    
+    result = gtk_dialog_run(GTK_DIALOG(dlg->dlg_xconfig_file));
+    gtk_widget_hide(dlg->dlg_xconfig_file);
+    
+    switch (result) {
+    case GTK_RESPONSE_ACCEPT:
+    case GTK_RESPONSE_OK:
+        filename = gtk_file_selection_get_filename
+            (GTK_FILE_SELECTION(dlg->dlg_xconfig_file));
+
+        gtk_entry_set_text(GTK_ENTRY(dlg->txt_xconfig_file), filename);
+
+        update_xconfig_save_buffer(dlg);
+        break;
+    default:
+        return;
+    }
+
+} /* xconfig_file_clicked() */
+
+
+
+/** run_save_xconfig_dialog() ****************************************
+ *
+ * run_save_xconfig_dialog() - Takes care of running the "Save X
+ * Configuration File" dialog.  Generates the X config file by
+ * calling the registered callback and takes care of keeping
+ * track of the requested filename etc.
+ *
+ **/
+
+void run_save_xconfig_dialog(SaveXConfDlg *dlg)
+{
+    void *buf;
+    GtkTextIter buf_start, buf_end;
+    gchar *filename;
+    gchar *tmp_filename;
+
+    gint result;
+
+
+    /* Generate the X config file save buffer */
+    update_xconfig_save_buffer(dlg);
+
+
+    /* Show the save dialog */
+    gtk_window_set_transient_for
+        (GTK_WINDOW(dlg->dlg_xconfig_save),
+         GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(dlg->parent))));
+
+    gtk_widget_hide(dlg->box_xconfig_save);
+    gtk_window_resize(GTK_WINDOW(dlg->dlg_xconfig_save), 350, 1);
+    gtk_window_set_resizable(GTK_WINDOW(dlg->dlg_xconfig_save),
+                             FALSE);
+    gtk_button_set_label(GTK_BUTTON(dlg->btn_xconfig_preview),
+                         "Show preview...");
+    gtk_widget_show(dlg->dlg_xconfig_save);
+    result = gtk_dialog_run(GTK_DIALOG(dlg->dlg_xconfig_save));
+    gtk_widget_hide(dlg->dlg_xconfig_save);
+    
+    
+    /* Handle user's response */
+    switch (result)
+    {
+    case GTK_RESPONSE_ACCEPT:
+
+        /* Get the filename to write to */
+        tmp_filename =
+            (gchar *) gtk_entry_get_text(GTK_ENTRY(dlg->txt_xconfig_file));
+        filename = tilde_expansion(tmp_filename);
+        if (filename == tmp_filename) {
+            filename = g_strdup(tmp_filename);
+        }
+
+        /* Get the buffer to write */
+        gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(dlg->buf_xconfig_save),
+                                   &buf_start, &buf_end);
+        buf = (void *) gtk_text_buffer_get_text
+            (GTK_TEXT_BUFFER(dlg->buf_xconfig_save), &buf_start,
+             &buf_end, FALSE);
+        if (!buf) {
+            nv_error_msg("Failed to read X configuration buffer!");
+            break;
+        }
+
+        /* Save the X config file */
+        nv_info_msg("", "Writing X config file '%s'", filename);
+        save_xconfig_file(dlg, filename, (char *)buf, 0644);
+        g_free(buf);
+        g_free(filename);
+        break;
+        
+    case GTK_RESPONSE_REJECT:
+    default:
+        /* do nothing. */
+        break;
+    }
+
+} /* run_save_xconfig_dialog() */
+
+
+
+/** create_save_xconfig_dialog() *************************************
+ *
+ * Creates the "Save X Configuration" button widget
+ *
+ **/
+
+SaveXConfDlg *create_save_xconfig_dialog(GtkWidget *parent,
+                                         Bool merge_toggleable,
+                                         generate_xconfig_callback xconf_gen_func,
+                                         void *callback_data)
+{
+    SaveXConfDlg *dlg;
+    GtkWidget *hbox;
+    GtkWidget *hbox2;
+    gchar *filename;
+
+
+    dlg = (SaveXConfDlg *) malloc (sizeof(SaveXConfDlg));
+    if (!dlg) return NULL;
+
+    dlg->parent = parent;
+    dlg->xconf_gen_func = xconf_gen_func;
+    dlg->merge_toggleable = merge_toggleable;
+    dlg->callback_data = callback_data;
+
+    /* Setup the default filename */
+    filename = (gchar *) xconfigOpenConfigFile(NULL, NULL);
+    if (filename) {
+        filename = g_strdup(filename);
+    } else {
+        filename = g_strdup("");
+    }
+    xconfigCloseConfigFile();
+
+    if (!filename) {
+        free(dlg);
+        return NULL;
+    }
+
+    /* Create the dialog */
+    dlg->dlg_xconfig_save = gtk_dialog_new_with_buttons
+        ("Save X Configuration",
+         GTK_WINDOW(gtk_widget_get_parent(GTK_WIDGET(parent))),
+         GTK_DIALOG_MODAL |  GTK_DIALOG_DESTROY_WITH_PARENT |
+         GTK_DIALOG_NO_SEPARATOR,
+         GTK_STOCK_SAVE,
+         GTK_RESPONSE_ACCEPT,
+         GTK_STOCK_CANCEL,
+         GTK_RESPONSE_REJECT,
+         NULL);
+
+    gtk_dialog_set_default_response(GTK_DIALOG(dlg->dlg_xconfig_save),
+                                    GTK_RESPONSE_REJECT);
+
+    gtk_dialog_set_has_separator(GTK_DIALOG(dlg->dlg_xconfig_save), TRUE);
+
+    /* Create the preview button */
+    dlg->btn_xconfig_preview = gtk_button_new();
+    g_signal_connect(G_OBJECT(dlg->btn_xconfig_preview), "clicked",
+                     G_CALLBACK(xconfig_preview_clicked),
+                     (gpointer) dlg);
+
+    /* Create the preview text window & buffer */
+    dlg->txt_xconfig_save = gtk_text_view_new();
+    gtk_text_view_set_left_margin
+        (GTK_TEXT_VIEW(dlg->txt_xconfig_save), 5);
+
+    dlg->buf_xconfig_save = gtk_text_buffer_new(NULL);
+    gtk_text_view_set_buffer(GTK_TEXT_VIEW(dlg->txt_xconfig_save),
+                             GTK_TEXT_BUFFER(dlg->buf_xconfig_save));
+
+    dlg->scr_xconfig_save = gtk_scrolled_window_new(NULL, NULL);
+    gtk_scrolled_window_set_shadow_type
+        (GTK_SCROLLED_WINDOW(dlg->scr_xconfig_save), GTK_SHADOW_IN);
+
+    /* Create the filename text entry */
+    dlg->txt_xconfig_file = gtk_entry_new();
+    gtk_widget_set_size_request(dlg->txt_xconfig_file, 300, -1);
+    gtk_entry_set_text(GTK_ENTRY(dlg->txt_xconfig_file), filename);
+    g_signal_connect(G_OBJECT(dlg->txt_xconfig_file), "activate",
+                     G_CALLBACK(xconfig_update_buffer),
+                     (gpointer) dlg);
+
+    /* Create the filename browse button */
+    dlg->btn_xconfig_file = gtk_button_new_with_label("Browse...");
+    g_signal_connect(G_OBJECT(dlg->btn_xconfig_file), "clicked",
+                     G_CALLBACK(xconfig_file_clicked),
+                     (gpointer) dlg);
+    dlg->dlg_xconfig_file =
+        gtk_file_selection_new("Please select the X configuration file");
+
+    /* Create the merge checkbox */
+    dlg->btn_xconfig_merge =
+        gtk_check_button_new_with_label("Merge with existing file.");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg->btn_xconfig_merge),
+                                 TRUE);
+    gtk_widget_set_sensitive(dlg->btn_xconfig_merge, merge_toggleable);
+    g_signal_connect(G_OBJECT(dlg->btn_xconfig_merge), "toggled",
+                     G_CALLBACK(xconfig_update_buffer),
+                     (gpointer) dlg);
+
+
+    /* Pack the preview button */
+    hbox = gtk_hbox_new(FALSE, 0);
+    hbox2 = gtk_hbox_new(FALSE, 0);
+
+    gtk_box_pack_start(GTK_BOX(hbox), dlg->btn_xconfig_preview,
+                       FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->dlg_xconfig_save)->vbox),
+                       hbox, FALSE, FALSE, 5);
+
+    /* Pack the preview window */
+    hbox = gtk_hbox_new(TRUE, 0);
+
+    gtk_container_add(GTK_CONTAINER(dlg->scr_xconfig_save),
+                      dlg->txt_xconfig_save);
+    gtk_box_pack_start(GTK_BOX(hbox), dlg->scr_xconfig_save, TRUE, TRUE, 5);
+    gtk_box_pack_start
+        (GTK_BOX(GTK_DIALOG(dlg->dlg_xconfig_save)->vbox),
+         hbox,
+         TRUE, TRUE, 0);
+    dlg->box_xconfig_save = hbox;
+    
+    /* Pack the filename text entry and browse button */
+    hbox = gtk_hbox_new(FALSE, 0);
+    hbox2 = gtk_hbox_new(FALSE, 5);
+    
+    gtk_box_pack_end(GTK_BOX(hbox2), dlg->btn_xconfig_file, FALSE, FALSE, 0);
+    gtk_box_pack_end(GTK_BOX(hbox2), dlg->txt_xconfig_file, TRUE, TRUE, 0);
+    gtk_box_pack_end(GTK_BOX(hbox), hbox2, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->dlg_xconfig_save)->vbox),
+                       hbox,
+                       FALSE, FALSE, 5);
+    
+    /* Pack the merge checkbox */
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->dlg_xconfig_save)->vbox),
+                       dlg->btn_xconfig_merge,
+                       FALSE, FALSE, 5);
+    
+    gtk_widget_show_all(GTK_DIALOG(dlg->dlg_xconfig_save)->vbox);
+    
+    return dlg;
+
+} /* create_save_xconfig_button() */
