@@ -261,12 +261,16 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_STEREO_EYES_EXCHANGE);
     MAKE_SIGNAL(NV_CTRL_NO_SCANOUT);
     MAKE_SIGNAL(NV_CTRL_GVO_CSC_CHANGED_EVENT);
+    MAKE_SIGNAL(NV_CTRL_X_SERVER_UNIQUE_ID);
     MAKE_SIGNAL(NV_CTRL_PIXMAP_CACHE);
     MAKE_SIGNAL(NV_CTRL_PIXMAP_CACHE_ROUNDING_SIZE_KB);
     MAKE_SIGNAL(NV_CTRL_IS_GVO_DISPLAY);
     MAKE_SIGNAL(NV_CTRL_PCI_ID);
     MAKE_SIGNAL(NV_CTRL_GVO_FULL_RANGE_COLOR);
+    MAKE_SIGNAL(NV_CTRL_SLI_MOSAIC_MODE_AVAILABLE);
     MAKE_SIGNAL(NV_CTRL_GVO_ENABLE_RGB_DATA);
+    MAKE_SIGNAL(NV_CTRL_IMAGE_SHARPENING_DEFAULT);
+    MAKE_SIGNAL(NV_CTRL_FRAMELOCK_SYNC_DELAY_RESOLUTION);
 
 #undef MAKE_SIGNAL
     
@@ -277,7 +281,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_GVO_ENABLE_RGB_DATA
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_FRAMELOCK_SYNC_DELAY_RESOLUTION
 #warning "There are attributes that do not emit signals!"
 #endif
 
