@@ -154,6 +154,7 @@ GtkWidget* ctk_gpu_new(
 
     gchar *screens;
     gchar *displays;
+    gchar *tmp_str;
 
     unsigned int display_devices;
     int xinerama_enabled;
@@ -269,52 +270,58 @@ GtkWidget* ctk_gpu_new(
         irq = g_strdup_printf("%d", tmp);
     }
     
-    /* List of X Screens using GPU */
+    /* List of X Screens using the GPU */
 
-    if (xinerama_enabled) {
+    screens = NULL;
+    ret = NvCtrlGetBinaryAttribute(handle,
+                                   0,
+                                   NV_CTRL_BINARY_DATA_XSCREENS_USING_GPU,
+                                   (unsigned char **)(&pData),
+                                   &len);
+    if (ret == NvCtrlSuccess) {
+        if (pData[0] == 0) {
+            screens = g_strdup("None");
+        } else {
+            NvCtrlAttributeHandle *screen_handle;
 
-        /* In Xinerama, there is only one logical X screen */
-
-        screens = g_strdup("Screen 0 (Xinerama)");
-
-    } else {
-        gchar *tmp_str;
-        screens = NULL;
-
-        ret = NvCtrlGetBinaryAttribute(handle,
-                                       0,
-                                       NV_CTRL_BINARY_DATA_XSCREENS_USING_GPU,
-                                       (unsigned char **)(&pData),
-                                       &len);
-        if (ret == NvCtrlSuccess) {
-            for (i = 1; i <= pData[0]; i++) {
-                
-                if (screens) {
-                    tmp_str = g_strdup_printf("%s,\nScreen %d",
-                                              screens, pData[i]);
-                } else {
-                    tmp_str = g_strdup_printf("Screen %d", pData[i]);
-                }
-                g_free(screens);
-                screens = tmp_str;
-            }
-            if (!screens) {
-                screens = g_strdup("None");
-
-            } else if (pData[0] > 0) {
-
-                ret = NvCtrlGetAttribute(t[pData[1]].h,
-                                         NV_CTRL_SHOW_SLI_HUD,
-                                         &tmp);
-
-                if (ret == NvCtrlSuccess) {
-                    tmp_str = g_strdup_printf("%s (SLI)", screens);
+            if (xinerama_enabled) {
+                screens = g_strdup("Screen 0 (Xinerama)");
+                /* XXX Use the only screen handle we have.
+                 *     This is currently OK since we only
+                 *     query xinerama attributes with this
+                 *     handle below.  If we needed to query
+                 *     a screen-specific attribute below,
+                 *     then we would need to get a handle
+                 *     for the correct screen instead.
+                 */
+                screen_handle = t[0].h;
+            } else {
+                for (i = 1; i <= pData[0]; i++) {
+                    if (screens) {
+                        tmp_str = g_strdup_printf("%s,\nScreen %d",
+                                                  screens, pData[i]);
+                    } else {
+                        tmp_str = g_strdup_printf("Screen %d", pData[i]);
+                    }
                     g_free(screens);
                     screens = tmp_str;
                 }
+                screen_handle = t[pData[1]].h;
+            }
+
+            ret = NvCtrlGetAttribute(screen_handle,
+                                     NV_CTRL_SHOW_SLI_HUD,
+                                     &tmp);
+            if (ret == NvCtrlSuccess) {
+                tmp_str = g_strdup_printf("%s (SLI)", screens);
+                g_free(screens);
+                screens = tmp_str;
             }
-            XFree(pData);
         }
+        XFree(pData);
+    }
+    if (!screens) {
+        screens = g_strdup("Unknown");
     }
 
     /* List of Display Device connected on GPU */
