@@ -167,6 +167,16 @@ typedef struct {
     /* whether this target type is aware of display devices */
     int uses_display_devices;
     
+    /*
+     * the minimum NV-CONTROL Protocol version required to use this target
+     * type; note that all future target types should be able to use 1.18,
+     * since that version and later allows NV-CONTROL clients to query the
+     * count of TargetTypes not recognized by the X server
+     */
+
+    int major;
+    int minor;
+
 } TargetTypeEntry;
 
 
@@ -287,7 +297,8 @@ void nv_parsed_attribute_add(ParsedAttribute *head, ParsedAttribute *a);
 void nv_parsed_attribute_free(ParsedAttribute *p);
 void nv_parsed_attribute_clean(ParsedAttribute *p);
 
-char *nv_get_attribute_name(const int attr);
+const char *nv_get_attribute_name(const int attr, const int flagsMask,
+                                  const int flags);
 
 char *nv_standardize_screen_name(const char *display_name, int screen);
 
